const fs = require('fs')
const path = require('path')
const chalk = require('chalk')

/**
 * 扩展对象属性
 * @param dest
 * @param src
 */
function extend (target, ...src) {
  if (typeof Object.assign === 'function') {
    Object.assign(target, ...src)
  }
  else {
    const first = src.shift()
    // 覆盖旧值
    for (const key in first) {
      target[key] = first[key]
    }
    if (src.length) {
      extend(target, ...src)
    }
  }
  return target
}

/**
 * 合并数组属性
 * @param dest
 * @param src
 */
function merge (target, ...src) {
  if (src.length) {
    src.forEach(item => {
      target = target.concat(item)
    })
  }
  return target
}

/**
 * -xxx-xxx转换为XxxXxx
 * @param value
 * @returns {void|string|XML|*}
 */
function hyphenedToCamelCase (value) {
  return value.replace(/-([a-z])/g, function (s, m) {
    return m.toUpperCase()
  })
}

/**
 * XxxXxx转换为-xxx-xxx
 * @param value
 * @returns {void|string|XML|*}
 */
function camelCaseToHyphened (value) {
  return value.replace(/([A-Z])/g, function (s, m) {
    return '-' + m.toLowerCase()
  })
}

/**
 * 判断是否为对象
 */
const toString = Object.prototype.toString
const OBJECT_STRING = '[object Object]'
function isPlainObject (obj) {
  return toString.call(obj) === OBJECT_STRING
}

/**
 * get datetime
 * @param  {Date} date []
 * @return {String}      []
 */
function datetime (date = new Date(), format = 'YYYY-MM-DD HH:mm:ss') {
  const fn = d => {
    return ('0' + d).slice(-2)
  }

  const d = new Date(date)
  const formats = {
    YYYY: d.getFullYear(),
    MM: fn(d.getMonth() + 1),
    DD: fn(d.getDate()),
    HH: fn(d.getHours()),
    mm: fn(d.getMinutes()),
    ss: fn(d.getSeconds())
  }

  return format.replace(/([a-z])\1+/ig, a => {
    return formats[a] || a
  })
}

/**
 * check path is exist
 */
function isExist (dir) {
  dir = path.normalize(dir)
  try {
    fs.accessSync(dir, fs.R_OK)
    return true
  }
  catch (e) {
    return false
  }
}

/**
 * check filepath is file
 */
function isFile (filePath) {
  if (!isExist(filePath)) return false
  try {
    const stat = fs.statSync(filePath)
    return stat.isFile()
  }
  catch (e) {
    return false
  }
}

/**
 * check path is directory
 */
function isDirectory (filePath) {
  if (!isExist(filePath)) return false
  try {
    const stat = fs.statSync(filePath)
    return stat.isDirectory()
  }
  catch (e) {
    return false
  }
}

/**
 * change path mode
 * @param  {String} p    [path]
 * @param  {String} mode [path mode]
 * @return {Boolean}      []
 */
function chmod (p, mode = '0777') {
  if (!isExist(p)) return false
  try {
    fs.chmodSync(p, mode)
    return true
  }
  catch (e) {
    return false
  }
}

/**
 *
 * @param dirname
 * @returns {boolean}
 */
function mkdirsSync (dirname) {
  if (fs.existsSync(dirname)) {
    return true
  }
  else {
    if (mkdirsSync(path.dirname(dirname))) {
      fs.mkdirSync(dirname)
      return true
    }
  }
}

/**
 * get files in path
 * @param  {} dir    []
 * @param  {} prefix []
 * @return {}        []
 */
function getdirFiles (dir) {
  dir = path.normalize(dir)
  if (!fs.existsSync(dir)) return []
  const files = fs.readdirSync(dir)
  let result = []
  files.forEach(item => {
    const currentDir = path.join(dir, item)
    const stat = fs.statSync(currentDir)
    if (stat.isFile()) {
      result.push(path.join(dir, item))
    }
    else if (stat.isDirectory()) {
      const cFiles = getdirFiles(currentDir, path.join(dir, item))
      result = result.concat(cFiles)
    }
  })
  return result
};

const logLevelMap = {}
function prependLevel (levelName, args) {
  !logLevelMap[levelName] && ((levelNameInner) => {
    const logLevel = levelNameInner.toUpperCase().substr(0, 4)
    logLevel.paddEnd && logLevel.paddEnd(4)
    logLevelMap[levelName] = logLevel
  })(levelName)

  if (typeof args[0] === 'string' && args[0].length > 1 && args[0][0] !== '[') {
    args[0] = `[${logLevelMap[levelName]}] ${args[0]}`
  }
}

/**
 * 带颜色的info, log, warn, error, trace的输出工具函数
 */
const colorconsole = {
  trace (...args) {
    prependLevel(`trace`, args)
    console.trace(...args)
  },
  log (...args) {
    prependLevel(`log`, args)
    console.log(chalk.green(...args))
  },
  info (...args) {
    prependLevel(`info`, args)
    console.info(chalk.green(...args))
  },
  warn (...args) {
    prependLevel(`warn`, args)
    console.warn(chalk.yellow.bold(...args))
  },
  error (...args) {
    prependLevel(`error`, args)
    console.error(chalk.red.bold(...args))
  },
  throw (...args) {
    throw new Error(chalk.red.bold(...args))
  }
}

module.exports = {
  extend: extend,
  hyphenedToCamelCase: hyphenedToCamelCase,
  camelCaseToHyphened: camelCaseToHyphened,
  merge: merge,
  isPlainObject: isPlainObject,
  isExist: isExist,
  isFile: isFile,
  datetime: datetime,
  isDirectory: isDirectory,
  chmod: chmod,
  mkdirsSync: mkdirsSync,
  getdirFiles: getdirFiles,
  colorconsole: colorconsole
}
